@@ -172,9 +172,9 @@ set_xinput_mouse_values (gboolean right_handed)
 {
     XDeviceInfo *device_info;
     unsigned char *buttons;
-    gint n_buttons, n_devices, i;
+    gint n_buttons, n_devices, i, j;
     gint idx_1 = 0, idx_3 = 1;
-    gsize buttons_capacity = 16;
+    gsize buttons_capacity = DEFAULT_PTR_MAP_SIZE;
 
     device_info = XListInputDevices (GDK_DISPLAY (), &n_devices);
 
@@ -204,22 +204,23 @@ set_xinput_mouse_values (gboolean right_handed)
             continue;
         }
 
-        n_buttons = XGetDeviceButtonMapping (GDK_DISPLAY (), device, buttons, DEFAULT_PTR_MAP_SIZE);
-        if (n_buttons > DEFAULT_PTR_MAP_SIZE)
+        n_buttons = XGetDeviceButtonMapping (GDK_DISPLAY (), device, buttons, buttons_capacity);
+        if (n_buttons > buttons_capacity)
         {
-            buttons = g_alloca (n_buttons);
-            n_buttons = XGetDeviceButtonMapping (GDK_DISPLAY (), device, buttons, n_buttons);
+            buttons = g_realloc (buttons, n_buttons);
+            buttons_capacity = n_buttons;
+            n_buttons = XGetDeviceButtonMapping (GDK_DISPLAY (), device, buttons, buttons_capacity);
         }
 
-        for (i = 0; i < n_buttons; i++)
+        for (j = 0; j < n_buttons; j++)
         {
-            if (buttons[i] == 1)
+            if (buttons[j] == 1)
             {
-                idx_1 = i;
+                idx_1 = j;
             }
-            else if (buttons[i] == ((n_buttons < 3) ? 2 : 3))
+            else if (buttons[j] == ((n_buttons < 3) ? 2 : 3))
             {
-                idx_3 = i;
+                idx_3 = j;
             }
         }
 
